/*jslint browser: true*/
/*global console, Framework7, alert, Dom7, Swiper, Template7*/

/**
 * A plugin for Framework7 to show a slideable welcome screen
 *
 * @module Framework7/prototype/plugins/welcomescreen
 * @author www.timo-ernst.net
 * @license MIT
 */
Framework7.prototype.plugins.welcomescreen = function(object, globalPluginParams) {
    'use strict';
    // Variables in module scope
    var $$ = Dom7,
        t7 = Template7,
        Welcomescreen;

    // Click handler to close welcomescreen
    $$(document).on('click', '.close-welcomescreen', function(e) {
        e.preventDefault();
        var $wscreen = $$(this).parents('.welcomescreen-container');
        if ($wscreen.length > 0 && $wscreen[0].f7Welcomescreen) {
            $wscreen[0].f7Welcomescreen.close();
        }
    });

    /**
     * Represents the welcome screen
     *
     * @class
     * @memberof module:Framework7/prototype/plugins/welcomescreen
     */
    Welcomescreen = function(slides, options) {

        // Private properties
        var self = this,
            defaultTemplate,
            template,
            container,
            swiperWelcome,
            swiperContainer,
            defaults = {
                closeButton: true, // enabled/disable close button
                closeButtonText: '', // close button text 跳过
                cssClass: '', // additional class on container
                button: true, // login and register button
                pagination: true, // swiper pagination
                loop: false, // swiper loop
                open: true // open welcome screen on init
            };

        /**
         * Initializes the swiper
         *
         * @private
         */
        function initSwiper() {
            swiperWelcome = new Swiper('.welcomescreen-container .swiper-container', {
                direction: 'horizontal',
                loop: options.loop,
                pagination: options.pagination ? swiperContainer.find('.swiper-pagination') : undefined
            });
        }

        /**
         * Sets colors from options
         *
         * @private
         */
        function setColors() {
            if (options.bgcolor) {
                container.css({
                    'background-color': options.bgcolor,
                    'color': options.fontcolor
                });
            }
        }

        /**
         * Sets the default template
         *
         * @private
         */
        function defineDefaultTemplate() {
            defaultTemplate = '<div class="welcomescreen-container {{#if options.cssClass}}{{options.cssClass}}{{/if}}">' +
                '{{#if options.closeButton}}' +
                '<div class="welcomescreen-closebtn close-welcomescreen">{{options.closeButtonText}}</div>' +
                '{{/if}}' +
                '<div class="welcomescreen-swiper swiper-container">' +
                '<div class="swiper-wrapper">' +
                '{{#each slides}}' +
                '<div class="swiper-slide" {{#if id}}id="{{id}}"{{/if}}>' +
                '{{#if content}}' +
                '<div class="welcomescreen-content">{{content}}</div>' +
                '{{else}}' +
                '{{#if picture}}' +
                '<div class="welcomescreen-picture">{{picture}}</div>' +
                '{{/if}}' +
                '{{#if text}}' +
                '<div class="welcomescreen-text">{{text}}</div>' +
                '{{/if}}' +
                '{{/if}}' +
                '</div>' +
                '{{/each}}' +
                '</div>' +
                '{{#if options.button}}' +
                '<div class="welcomescreen-text"><div class="content-block"><div class="row"><div class="col-33"></div><div class="col-33">'+
                '<a href="#" class="link open-popup" data-popup=".demo-screen">快速体验</a></div><div class="col-33"></div></div></div>'+
                '<div class="content-block"><div class="row">'+
                '<div class="col-50"><a href="#" class="button button-big active open-popup button-fill button-raised" data-popup=".signin-screen">登录</a></div>'+
                '<div class="col-50"><a href="#" class="button button-big action-register button-raised" data-popup=".register-screen">注册</a></div>'+
                '</div></div></div>'+
                '{{/if}}' +
                '{{#if options.pagination}}' +
                '<div class="welcomescreen-pagination swiper-pagination"></div>' +
                '{{/if}}' +
                '</div>' +
                '</div>';
        }

        /**
         * Sets the options that were required
         *
         * @private
         */
        function applyOptions() {
            var def;
            options = options || {};
            for (def in defaults) {
                if (typeof options[def] === 'undefined') {
                    options[def] = defaults[def];
                }
            }
        }

        /**
         * Compiles the template
         *
         * @private
         */
        function compileTemplate() {
            if (!options.template) {
                // Cache compiled templates
                if (!object._compiledTemplates.welcomescreen) {
                    object._compiledTemplates.welcomescreen = t7.compile(defaultTemplate);
                }
                template = object._compiledTemplates.welcomescreen;
            } else {
                template = t7.compile(options.template);
            }
        }

        /**
         * Shows the welcome screen
         *
         * @public
         * @memberof module:Framework7/prototype/plugins/welcomescreen
         */
        self.open = function() {
            container = $$(template({
                options: options,
                slides: slides
            }));
            swiperContainer = container.find('.welcomescreen-container .swiper-container');
            setColors();
            $$('body').prepend(container);
            initSwiper();
            container[0].f7Welcomescreen = self;
            if (typeof options.onOpened === 'function') {
                options.onOpened();
            }
        };

        /**
         * Hides the welcome screen
         *
         * @public
         * @memberof module:Framework7/prototype/plugins/welcomescreen
         */
        self.close = function() {
            if (swiperWelcome) {
                swiperWelcome.destroy(true);
            }
            if (container) {
                container.remove();
            }
            container = swiperContainer = swiperWelcome = undefined;
            if (typeof options.onClosed === 'function') {
                options.onClosed();
            }
        };

        /**
         * Shows the next slide
         *
         * @public
         * @memberof module:Framework7/prototype/plugins/welcomescreen
         */
        self.next = function() {
            if (swiperWelcome) {
                swiperWelcome.slideNext();
            }
        };

        /**
         * Shows the previous slide
         *
         * @public
         * @memberof module:Framework7/prototype/plugins/welcomescreen
         */
        self.previous = function() {
            if (swiperWelcome) {
                swiperWelcome.slidePrev();
            }
        };

        /**
         * Goes to the desired slide
         *
         * @param {number} index The slide to show
         * @public
         * @memberof module:Framework7/prototype/plugins/welcomescreen
         */
        self.slideTo = function(index) {
            if (swiperWelcome) {
                swiperWelcome.slideTo(index);
            }
        };

        /**
         * Initialize the instance
         *
         * @method init
         */
        (function() {
            defineDefaultTemplate();
            compileTemplate();
            applyOptions();

            // Open on init
            if (options.open) {
                self.open();
            }

        }());

        // Return instance
        return self;
    };

    object.welcomescreen = function(slides, options) {
        return new Welcomescreen(slides, options);
    };
};
